//*****************************************************************************
//*****************************************************************************
//  FILENAME:   sensor.c
//  Version: 1.0, 2007/03/08
//  Generated by Andy Kiser
//
//  DESCRIPTION:  sensor specific functions 
//
//-----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
//*****************************************************************************
//*****************************************************************************
//#define __DEBUG__
#include <m8c.h>        // part specific constants and macros
#include "sensor.h"
#include "psocgpioint.h"
#include "feature.h"
#include "parameters.h"
#include "types_n_macros.h"
#include "sht1x.h"
//----------------------------------------
// Sensor parameters
// Factory Defaults are defined here (NUM_xxx are defined in "sensor.h")
//----------------------------------------
const BYTE FactoryDefault_Param_8   [NUM_PARAM_8] =
{
	0x00,		// report mode
	0x00,		// LED mode
	0x00		// Sensor status, specific for SHT1x
};

const WORD FactoryDefault_Param_16  [NUM_PARAM_16] = 
{
	500,		// report rate [ms]
	500,		// sample rate [ms]
	50000		// sample rate [us]	(not used for this sensor)
};

const DWORD FactoryDefault_Param_32  [NUM_PARAM_32] =
{
	0x00000000	// currently no 32bit parameters
};

const tParam_168 FactoryDefault_Param_168 [NUM_PARAM_168] =
{
	"Humidity Sensor",	// user device name
	"Frame Number",		// user name Channel 0
	"Humidity",			// user name Channel 1
	"Temperature"		// user name Channel 2
};


//----------------------------------------
// this module's global variables
//----------------------------------------
tReport Report;

//----------------------------------------
// constant lookup-table for conversion ADC->physical value
//----------------------------------------


//----------------------------------------
// type definitions
//----------------------------------------

typedef enum 
{	
	SHT1x_PreReset, 
	SHT1x_Resetting, 
	SHT1x_Measuring_Temp,  
	SHT1x_Measuring_Humid, 
	SHT1x_Status, 
	SHT1x_Idle
} tFSM_Measure;

//----------------------------------------
// variables

tFSM_Measure	FSM;
WORD			Countdown_1ms;

BOOL			NewSample = FALSE;

// store original drivemodes during suspend
BYTE			DM0_2, DM0_1, DM0_0;
BYTE			DM2_2, DM2_1, DM2_0;

//------------------------------------------------------
// Commands for Sensirion SHT1x sensors
//------------------------------------------------------
// the complete command byte consists of 3 address bits, followed by 5 command bits
// the only address accepted is %000
// --> transmission sequence: a2-a1-a0-c4-c3-c2-c1-c0
//     (ax = address bit x / cx = command bit x)
#define cmdSHT1x_MeasureTemp	0x03	// %00011
#define cmdSHT1x_MeasureHumid	0x05	// %00101
#define cmdSHT1x_ReadStatus		0x07	// %00111
#define cmdSHT1x_WriteStatus	0x06	// %00110
#define cmdSHT1x_Reset			0x1E	// %11110



//----------------------------------------
// function prototypes

//-----------------------------------------------------------------------------
// Initializes the Sensor 
//-----------------------------------------------------------------------------
void Sensor_Start ()
{
	FSM = SHT1x_PreReset;
	Countdown_1ms = 12;	// after Power-up, wait at least 11ms before sending any command to the SHT1x sensor
}

//-----------------------------------------------------------------------------
// Prepare Sensor for Suspend
//-----------------------------------------------------------------------------
void Sensor_Suspend()
{

//  Reset the Sensor to assure that it does not drive the DATA signal
	SHT1x_Start();
// call assembler routine
//	SHT1x_Suspend();
}
//-----------------------------------------------------------------------------
// Resume Sensor from Suspend mode
//-----------------------------------------------------------------------------
void Sensor_Resume()
{
	// call assembler routine
	SHT1x_Resume();
	Sensor_Start();
}

//-----------------------------------------------------------------------------
// static WORD CalcTemperature (WORD RawTemp)
//
//	Convert value read from the SHT1x sensor into an absolute temperature
//-----------------------------------------------------------------------------
// ARGUMENTS:
//	RawTemp:	Raw Temperature value (14bit) as read from the sensor
// RETURNS:
//	Temperature in [1E-2 K] 
//	e.g. 27'314 = 273.14K = 0C
// SIDE EFFECTS: 
//	none
//-----------------------------------------------------------------------------
static WORD CalcTemperature (WORD RawTemp)
{
	return 23350 + RawTemp;
}
//-----------------------------------------------------------------------------
// static WORD CalcHumidity (WORD RawHumidity)
//
//	Convert value read from the SHT1x sensor into a relative humidity
//  Humidity(12bit): 10^4*RH = -400 + 4.05*ADC-0.00028*ADC^2
//-----------------------------------------------------------------------------
// ARGUMENTS:
//	RawHumidity:	Raw Humidity value (12bit) as read from the sensor
// RETURNS:
//	Humidity [1e-4]
//  e.g. 3'392 = 0.3392 = 33.92% relative humidity
// SIDE EFFECTS: 
//	none
//-----------------------------------------------------------------------------
#define c_4_05_SHL13	33178
#define c_0_00028_SHL23	2349

static WORD CalcHumidity (WORD RawHumidity)
{
	DWORD Humid_Tmp1;
	DWORD Humid_Tmp2;
	
	Humid_Tmp1 = RawHumidity * c_4_05_SHL13;
	Humid_Tmp1 = Humid_Tmp1 >> 13;
	
	Humid_Tmp2 = RawHumidity;
	Humid_Tmp2 *= RawHumidity;
	Humid_Tmp2 = Humid_Tmp2 >> 12;
	Humid_Tmp2 *= c_0_00028_SHL23;
	Humid_Tmp2 = Humid_Tmp2 >> 11;
	
	return (Humid_Tmp1 - Humid_Tmp2 - 400);
}

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// cannot be handled with the standard functionality in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//
//	return Value: number of Parameter bytes (1-31) that are 
//				  returned in the IN report
//-----------------------------------------------------------------------------
BYTE Sensor_Specific_Feature(void)
{
	// currently no special feature reports are handled
	return 0;
}

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// sets / gets a 8/16/32/168bit parameter in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//-----------------------------------------------------------------------------
void Sensor_Specific_Parameter(BYTE *NumberOfReturnedBytes)
{
	// no special parameter handling necessary
}


//-----------------------------------------------------------------------------
// void Sensor_FSM_Measure(void)
//
//	finite state machine to control the measurement process for 
//  the sensor
//
//  IMPORTANT: 	call this routine once every millisecond. Otherwise the timing
//				parameters cannot be met.
//-----------------------------------------------------------------------------
void Sensor_FSM_Measure() 
{
	       BYTE SHT1x_Status_ToBe; // status that is set by the user
	static BYTE	SHT1x_Status_Readback = 0x00; // status read back from the sensor
	static WORD	Temp_Raw,
				Temp,
				Humid_Raw,
				Humid;

	// decrement millisecond counter
	if (Countdown_1ms != 0)
		Countdown_1ms--;

	// process actual state machine
	switch (FSM)
	{
		case SHT1x_PreReset: 
			if (Countdown_1ms == 0)
			{
				// do a soft reset, then wait at least 11ms before proceeding
				FSM = SHT1x_Resetting;
				SHT1x_Start();
				Countdown_1ms = 12;
			}
			break;			
			
		case SHT1x_Resetting:
			if (Countdown_1ms == 0)
			{
				// start a temperature measurement.
				CRC8 = 0;
				FSM = SHT1x_Status;								
			}
			break;			
		
		case SHT1x_Measuring_Temp:
			if (SHT1x_nbMeasure_Done())
			{
				// temperature measurement is done
				// => process the result
				Temp_Raw = SHT1x_nbMeasure_Result();

				if (CRC8 == 0)
				{
					// convert 12bit -> 14bit if necessary
					if ( (SHT1x_Status_Readback & 1) == 1 )
						Temp_Raw <<= 2;
					//Temperature(14bit): 10^2*T = 23350 + ADC
					Temp = CalcTemperature(Temp_Raw);  // 
					// temperature measurement processed
					// => start humidity measurement
					CRC8 = (SHT1x_Status_Readback & 0x0F); Mirror_CRC8();
					SHT1x_nbMeasure_Start(cmdSHT1x_MeasureHumid);	

					FSM = SHT1x_Measuring_Humid;
				}
				else
					FSM = SHT1x_PreReset;	
			}
			break;
			
		case SHT1x_Measuring_Humid: 
			if (SHT1x_nbMeasure_Done())
			{
				// humidity measurement is done
				// => process the result
				Humid_Raw = SHT1x_nbMeasure_Result();
				if (CRC8 == 0)
				{
					FSM = SHT1x_Status;
					
					// convert 8bit -> 12bit if necessary
					if ( (SHT1x_Status_Readback & 1) == 1 )
						Humid_Raw <<= 4;
					//Humidity(12bit): 10^4*RH = -400 + 4.05*ADC-0.00028*ADC^2
					Humid = CalcHumidity(Humid_Raw);
					if (Humid <0)
						Humid = 0;
					if (Humid > 10000)
						Humid = 10000;
					
					
					// fill measured data into IN REPORT
					Report.Ch1_LSB = Humid & 0xFF;
					Report.Ch1_MSB = Humid / 0x100;
					Report.Ch2_LSB = Temp & 0xFF;
					Report.Ch2_MSB = Temp / 0x100;
					
					NewSample=TRUE;
				}
				else
					FSM = SHT1x_PreReset;	
			}
			break;
		
		case SHT1x_Status:
			// set new status if requested 
			Get_Param_8(PAR8_SENSORSTATUS, VOLATILE, &SHT1x_Status_ToBe);
			
			if (SHT1x_Status_Readback != SHT1x_Status_ToBe)
				SHT1x_WriteStatus(SHT1x_Status_ToBe);
			// read back sensor status
			SHT1x_Status_Readback = SHT1x_ReadStatus();
		
			FSM = SHT1x_Idle;
			break;

		case SHT1x_Idle:
			if (Countdown_1ms == 0)
			{
				// reset samplerate counter
				Get_Param_16(PAR16_SAMPLERATE_MS, VOLATILE, &Countdown_1ms);

				// start a temperature measurement.
				CRC8 = (SHT1x_Status_Readback & 0x0F); Mirror_CRC8();
				SHT1x_nbMeasure_Start(cmdSHT1x_MeasureTemp);

				FSM = SHT1x_Measuring_Temp;
			}
			break;			
	}
}

//-----------------------------------------------------------------------------
void Sensor_Debug(void)
{
/* currently no debug function */
}