;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME:   SHT1x.asm
;;  Version: 1.0, 2006/08/07 at 17:13
;;  Generated by Andy Kiser
;;
;;  DESCRIPTION:  User module for access to the Sensirion SHT1x sensor
;;                for the 22/24/25/26/27xxx PSoC family of devices.
;;
;;
;;  NOTE: User Module APIs conform to the fastcall16 convention for marshalling
;;        arguments and observe the associated "Registers are volatile" policy.
;;        This means it is the caller's responsibility to preserve any values
;;        in the X and A registers that are still needed after the API functions
;;        returns. For Large Memory Model devices it is also the caller's 
;;        responsibility to perserve any value in the CUR_PP, IDX_PP, MVR_PP and 
;;        MVW_PP registers. Even though some of these registers may not be modified
;;        now, there is no guarantee that will remain the case in future releases.
;;-----------------------------------------------------------------------------
;;  Copyright (c) Toradex AG 2006. All Rights Reserved.
;;*****************************************************************************
;;*****************************************************************************

;; Extract of the sensor's datasheet
;;-----------------------------------------------------------------------------
;; - The sensor's digital interface can be accessed at random frequency,
;;   up to 1MHz.
;;   For most accesses this means that data transfer can be done at full speed,
;;   provided that the protocol is realized in software
;;-----------------------------------------------------------------------------
;; - Transmit Data
;;   Data must be stable while SCK='H'
;; - Receive Data
;;   data changes after the falling edge of SCK
;;-----------------------------------------------------------------------------
;; - Soft Reset Command
;;   Wait for at least 11ms after Soft Reset Command!
;;-----------------------------------------------------------------------------
;; - Sampling Rate
;;   The SHT1x takes approximately 11-250ms to finish a measurement
;;-----------------------------------------------------------------------------
;; - self heating
;;   to avoid self-heating, the sensor should not be active for more than 10%
;;   of the time 
;;   --> only 1 measurement every 2sec. for 14bit resolution!
;;-----------------------------------------------------------------------------
;; - Read Status Command
;;   Ignore Checksum when reading status!
;;	 Sensirion confirmed there is an issue with that: maybe the status register
;;	 itself should be the initial value for CRC8.
;;-----------------------------------------------------------------------------

include "memory.inc"
include "m8c.inc"


export  SHT1x_Start
export  _SHT1x_Start
export	SHT1x_Suspend
export	_SHT1x_Suspend
export	SHT1x_Resume
export	_SHT1x_Resume
export	SHT1x_Measure
export	_SHT1x_Measure
export	SHT1x_ReadStatus
export	_SHT1x_ReadStatus
export	SHT1x_WriteStatus
export	_SHT1x_WriteStatus

export	SHT1x_nbMeasure_Start
export	_SHT1x_nbMeasure_Start
export	SHT1x_nbMeasure_Done
export	_SHT1x_nbMeasure_Done
export	SHT1x_nbMeasure_Result
export	_SHT1x_nbMeasure_Result
export	Mirror_CRC8;
export	_Mirror_CRC8;


export	CRC8
export  _CRC8

;------------------------------------------------------
;  Register Address Constants for  SHT1x
;------------------------------------------------------
SHT1x_DR:			equ PRT2DR	; Data Register -> Port2
SHT1x_DM0:			equ PRT2DM0
SHT1x_DM1:			equ PRT2DM1
SHT1x_DM2:			equ PRT2DM2

;------------------------------------------------------
; Data Masks for  SHT1x
;------------------------------------------------------
SHT1x_SCK_Mask:   	equ  %00010000	;SCK  -> Port2[4]
SHT1x_DATA_Mask:	equ	 %01000000	;DATA -> Port2[6]

D_SCK_00:   		equ  %00000000
D_SCK_0H:   		equ  SHT1x_SCK_Mask
D_SCK_H0:   		equ  SHT1x_DATA_Mask
D_SCK_HH:   		equ  SHT1x_DATA_Mask | SHT1x_SCK_Mask

;------------------------------------------------------
; Commands for Sensirion SHT1x sensors
;------------------------------------------------------
; the complete command byte consists of 3 address bits, followed by 5 command bits
; the only address accepted is %000
; --> transmission sequence: a2-a1-a0-c4-c3-c2-c1-c0
;     (ax = address bit x / cx = command bit x)
cmdSHT1x_MeasureTemp:	equ	%00011
cmdSHT1x_MeasureHumid:	equ %00101
cmdSHT1x_ReadStatus:	equ %00111
cmdSHT1x_WriteStatus:	equ %00110
cmdSHT1x_Reset:			equ %11110


AREA InterruptRAM (RAM,REL,CON)
;-----------------------------------------------
; Variable Allocation
;-----------------------------------------------
CRC8:
_CRC8:	BLK 1

AREA UserModules (ROM, REL)

.SECTION
;-----------------------------------------------------------------------------
; void SHT1x_Start(void)
;
;	Initialize SHT1x interface after startup
;-----------------------------------------------------------------------------
; ARGUMENTS:
;	none
; RETURNS:
;	none
; SIDE EFFECTS: 
;	Registers are volatile: the A and X registers may be modified
;-----------------------------------------------------------------------------

_SHT1x_Start:
 SHT1x_Start:
	; initialize Port Mode
	; set port0[x] Drivemode to 110 (Hi-Z analog)
	; set port0[4] Drivemode to 101 (SCK, slow strong)
	; set port0[6] Drivemode to 011 (SDATA, pullup)
	M8C_SetBank1
	mov	reg[SHT1x_DM0], %01010000	
	mov	reg[SHT1x_DM1], %11101111	
	M8C_SetBank0
	mov	reg[SHT1x_DM2], %10111111	
	
	; please configure the device manager to assure, that 
	;  - the DATA port (by default PORT_2[4] is in "PullUp" configuration
	;  - the SCK port (by default PORT_2[6] is in "output" configuration
	mov reg[SHT1x_DR], SHT1x_DATA_Mask	;DATA=H, SCK=0
	
	mov		A, 0xFF		; toggle SCK >= 9 times to reset the connection interface
	call	SHT1x_TxByte
	mov		A, 0xFF
	call	SHT1x_TxByte
	
	call	SHT1x_TxStart		; Transmit Soft reset command
	mov		A, cmdSHT1x_Reset
	call	SHT1x_TxByte
	call	SHT1x_WaitForAck
	
	// sensor is resetting - 
	// wait 11ms before next command
	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
; void SHT1x_Suspend(void)
;
;	Prepare the SHT1x for suspend
;-----------------------------------------------------------------------------
; ARGUMENTS:
;	none
; RETURNS:
;	none
; SIDE EFFECTS: 
;	Registers are volatile: the A and X registers may be modified
;-----------------------------------------------------------------------------

_SHT1x_Suspend:
 SHT1x_Suspend:
	; set port0[x] Drivemode to 110 (Hi-Z analog)
	M8C_SetBank1
	mov	reg[SHT1x_DM0], %00000000	
	mov	reg[SHT1x_DM1], %11111111	
	M8C_SetBank0
	mov	reg[SHT1x_DM2], %11111111	
	
	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
; void SHT1x_Resume(void)
;
;	Resume the SHT1x from suspend
;-----------------------------------------------------------------------------
; ARGUMENTS:
;	none
; RETURNS:
;	none
; SIDE EFFECTS: 
;	Registers are volatile: the A and X registers may be modified
;-----------------------------------------------------------------------------

_SHT1x_Resume:
 SHT1x_Resume:
	; set port0[x] Drivemode to 110 (Hi-Z analog)
	; set port0[4] Drivemode to 101 (SCK, slow strong)
	; set port0[6] Drivemode to 011 (SDATA, pullup)
	M8C_SetBank1
	mov	reg[SHT1x_DM0], %01010000	
	mov	reg[SHT1x_DM1], %11101111	
	M8C_SetBank0
	mov	reg[SHT1x_DM2], %10111111	
	
	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
; void SHT1x_TxStart(void)
;
;	Transmit Start Command on SHT1x interface
;-----------------------------------------------------------------------------
; ARGUMENTS:
;	none
; RETURNS:
;	none
; SIDE EFFECTS: 
;	none
;-----------------------------------------------------------------------------

_SHT1x_TxStart:
 SHT1x_TxStart:
	mov reg[SHT1x_DR], D_SCK_HH		;DATA=H, SCK=1
	mov reg[SHT1x_DR], D_SCK_0H		;DATA=0, SCK=1
	mov reg[SHT1x_DR], D_SCK_00		;DATA=0, SCK=0
	mov reg[SHT1x_DR], D_SCK_0H		;DATA=0, SCK=1
	mov reg[SHT1x_DR], D_SCK_HH		;DATA=H, SCK=1

	mov reg[SHT1x_DR], D_SCK_H0    	;DATA=H, SCK=0
	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
; void SHT1x_TxByte (char DataByte)
;
;	Transmit 8 Bits through the SHT1x interface  
;-----------------------------------------------------------------------------
; ARGUMENTS:  ( DataByte )
;	Value to be shiftet out (MSB first)
; RETURNS:
;	none
; SIDE EFFECTS:
;	Registers are volatile: the A and X registers may be modified
;-----------------------------------------------------------------------------
_SHT1x_TxByte:
 SHT1x_TxByte:
 	; setup loop for 8 bits
	mov	X, 8

.TxBit:
	; shift next data bit into the Carry Flag (MSB first)
	; and set the DATA pin accordingly
	asl	A		 
	jnc	.Tx0
.Tx1:
	mov reg[SHT1x_DR], D_SCK_H0		;DATA=H, SCK=0
	mov reg[SHT1x_DR], D_SCK_HH		;DATA=H, SCK=1
	mov reg[SHT1x_DR], D_SCK_H0    	;DATA=H, SCK=0
	jmp	.Clk
.Tx0:
	mov reg[SHT1x_DR], D_SCK_00		;DATA=0, SCK=0
	mov reg[SHT1x_DR], D_SCK_0H		;DATA=0, SCK=1
	mov reg[SHT1x_DR], D_SCK_00    	;DATA=0, SCK=0

	; generate a clock pulse on the SCK pin
.Clk:
	
	; loop for each bit
	dec	X
	jnz	.TxBit

	; release DATA pin to high
	mov reg[SHT1x_DR], D_SCK_H0    	;DATA=H, SCK=0
	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
; void SHT1x_TxByte_withCRC (char DataByte)
;
;	Transmit 8 Bits through the SHT1x interface,
;   update CRC8 value
;-----------------------------------------------------------------------------
; ARGUMENTS:  ( DataByte )
;	Value to be shiftet out (MSB first)
; RETURNS:
;	none
; SIDE EFFECTS:
;	Registers are volatile: the A and X registers may be modified
;   Global variable CRC8 will be modified
;-----------------------------------------------------------------------------
_SHT1x_TxByte_withCRC:
 SHT1x_TxByte_withCRC:
	RAM_SETPAGE_CUR >CRC8	; Set page register for the global variable "CRC8"
	
 	; setup loop for 8 bits
	mov	X, 8
.TxBit:
	asl [CRC8]				; CRC calculation step1
	jnc	.Bit7was0			; CRC calculation step2a
	xor	[CRC8], %00110001	; CRC calculation step2b
.Bit7was0:


	; shift next data bit into the Carry Flag (MSB first)
	; and set the DATA pin accordingly
	asl	A		 
	jnc	.Tx0
.Tx1:
	xor [CRC8], %00110001	; CRC calculation step3

	mov reg[SHT1x_DR], D_SCK_H0		;DATA=H, SCK=0
	mov reg[SHT1x_DR], D_SCK_HH		;DATA=H, SCK=1
	mov reg[SHT1x_DR], D_SCK_H0		;DATA=H, SCK=0
	jmp	.Clk
.Tx0:
	mov reg[SHT1x_DR], D_SCK_00		;DATA=0, SCK=0
	mov reg[SHT1x_DR], D_SCK_0H		;DATA=0, SCK=1
	mov reg[SHT1x_DR], D_SCK_00		;DATA=0, SCK=0

	; generate a clock pulse on the SCK pin
.Clk:
	
	; loop for each bit
	dec	X
	jnz	.TxBit

	; release DATA pin to high
	mov reg[SHT1x_DR], D_SCK_H0		;DATA=H, SCK=0
	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
; char SHT1x_RxByte (void)
;
;	Receive 8 Bits through the SHT1x interface (no checksum, MSB first)
;-----------------------------------------------------------------------------
; ARGUMENTS: 
;	none
; RETURN VALUE
;	received data byte
; SIDE EFFECTS:
;	Registers are volatile: the X register may be modified
;-----------------------------------------------------------------------------
_SHT1x_RxByte:
 SHT1x_RxByte:
 	; setup loop for 8 bits
	mov	X, 8

.RxBit:
	; generate positive clock edge on SCK pin
	mov reg[SHT1x_DR], SHT1x_DATA_Mask | SHT1x_SCK_Mask	;DATA=H, SCK=1
	
	; shift next data bit into accumulator A (MSB first)
	asl A
	tst	reg[SHT1x_DR], SHT1x_DATA_Mask
	jz	.Rx0
.Rx1:
	inc A
.Rx0:
	; generate negative clock edge on SCK pin
	mov reg[SHT1x_DR], SHT1x_DATA_Mask                 	;DATA=H, SCK=0
	
	; loop for each bit
	dec	X
	jnz	.RxBit
	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
; char SHT1x_RxByte_withCRC (void)
;
;	Receive 8 Bits through the SHT1x interface (update CRC8, MSB first)
;	The CRC8 is using the Polynom x^8+x^5+x^4
;	the algorithm for updating the CRC8 with a single bit "IN" is:
;		Bit7* = Bit6
;		Bit6* = Bit5
;		Bit5* = Bit4 xor Bit7 xor "IN"
;		Bit4* = Bit3 xor Bit7 xor "IN"
;		Bit3* = Bit2
;		Bit2* = Bit1
;		Bit1* = Bit0
;		Bit0* = Bit7 xor "IN"
;-----------------------------------------------------------------------------
; ARGUMENTS:
;	none
; RETURN VALUE
;	received data byte
; SIDE EFFECTS:
;	Registers are volatile: the X register may be modified
;	Variable CRC8 is modified
;-----------------------------------------------------------------------------
_SHT1x_RxByte_withCRC:
 SHT1x_RxByte_withCRC:
	; CRC calculation is embedded in the data receive, to minimize
	; number of conditional branches

	RAM_SETPAGE_CUR >CRC8	; Set page register for the global variable "CRC8"

 	; setup loop for 8 bits
	mov	X, 8

.RxBit:
	; generate positive clock edge on SCK pin
	mov reg[SHT1x_DR], D_SCK_HH		;DATA=H, SCK=1
	
	; shift next data bit into accumulator A (MSB first)
	asl A

	asl [CRC8]				; CRC calculation step1
	jnc	.Bit7was0			; CRC calculation step2a
	xor	[CRC8], %00110001	; CRC calculation step2b
.Bit7was0:
	tst	reg[SHT1x_DR], SHT1x_DATA_Mask
	NOP
	NOP
	
	jz	.Rx0
.Rx1:
	inc A
	xor [CRC8], %00110001	; CRC calculation step3
.Rx0:
	
.Clk:
	; generate negative clock edge on SCK pin
	mov reg[SHT1x_DR], D_SCK_H0 	;DATA=H, SCK=0
	
	; loop for each bit
	dec	X
	jnz	.RxBit
	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
; unsigned void SHT1x_TxAck (void)
;
;	Sends an acknoledge on the bus. 
;-----------------------------------------------------------------------------
; ARGUMENTS: 
;	none
; RETURN VALUE
;	none
; SIDE EFFECTS:
;	none
;-----------------------------------------------------------------------------
_SHT1x_TxAck:
 SHT1x_TxAck:
 	; pull the DATA pin LOW
	mov reg[SHT1x_DR], D_SCK_00 	;DATA=0, SCK=0
 	
	; generate a clock pulse on the SCK pin
	mov reg[SHT1x_DR], D_SCK_0H		;DATA=0, SCK=1
	mov reg[SHT1x_DR], D_SCK_00		;DATA=0, SCK=0
	
	; release DATA pin to high
	mov reg[SHT1x_DR], D_SCK_H0		;DATA=H, SCK=0

	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
; unsigned void SHT1x_WaitForAck (void)
;
;	Waits until the slave sensor puts an acknoledge on the bus. Then 
;	clears the acknowledge by generating a clock cycle on the SCK pin.
;-----------------------------------------------------------------------------
; ARGUMENTS: 
;	none
; RETURN VALUE
;	none
; SIDE EFFECTS:
;	none
;-----------------------------------------------------------------------------
_SHT1x_WaitForAck:
 SHT1x_WaitForAck:
 
	; make sure that we do not drive the data signal low
	mov reg[SHT1x_DR], D_SCK_H0		;DATA=H, SCK=0
.loop_until_data_low :
 	; loop until DATA pin gets LOW
	tst		reg[SHT1x_DR], SHT1x_DATA_Mask
	jnz		.loop_until_data_low 
 	
	; generate a clock pulse on the SCK pin
	mov reg[SHT1x_DR], D_SCK_HH		;DATA=H, SCK=1
	mov reg[SHT1x_DR], D_SCK_H0		;DATA=H, SCK=0

	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
; unsigned void Mirror_CRC8 (void)
;
;	Mirrors the bit of the global CRC8 variable MSB...LSB  => LSB...MSB
;-----------------------------------------------------------------------------
; ARGUMENTS: 
;	none
; RETURN VALUE
;	none
; SIDE EFFECTS:
;	Registers are volatile: the A and X registers may be modified
;	CRC8 variable will be modified
;-----------------------------------------------------------------------------
_Mirror_CRC8:
 Mirror_CRC8:
 
 	RAM_SETPAGE_CUR >CRC8	; Set page register for the global variable "CRC8"
 
	mov	X, 8
.mirror_bit:	
	asl [CRC8]
	rrc	A
	
	dec	X			;loop 8 times
	jnz	.mirror_bit

	mov	[CRC8], A
	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
; unsigned int SHT1x_Measure (char command)
;
;	Gets the raw value for relative humidity or temperature from a 
;	Sensirion SHT1x sensor. 
;	the function hangs if the sensor does not respond properly (missing ACK,
;	wrong checksum)
;-----------------------------------------------------------------------------
; ARGUMENTS: 
;	command:	Address and command byte to send. Valid values are:
;					%00000011 (measure temperature) and
;					%00000101 (measure humidity)			
; RETURN VALUE
;	raw measured value  X=MSB, A=LSB
; SIDE EFFECTS:
;	CRC8 variable will be modified
;-----------------------------------------------------------------------------
_SHT1x_Measure:
 SHT1x_Measure:
 	RAM_SETPAGE_CUR >CRC8	; Set page register for the global variable "CRC8"

	mov		[CRC8], 0; ATTENTION: CRC8 should be initialized with (SHT1x_STATUS &0x0F)
 	call	SHT1x_nbMeasure_Start
 .Loop_Until_Measurement_Done:
	call	SHT1x_nbMeasure_Done
	 jz		.Loop_Until_Measurement_Done 
 
	call	SHT1x_nbMeasure_Result

;	repeat if wrong checksum	
;	cmp		[CRC8], 0
;	jnz		SHT1x_Measure
	
	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
; void SHT1x_WriteStatus (Status)
;
;	Writes the status register of a Sensirion SHT1x sensor
;-----------------------------------------------------------------------------
; ARGUMENTS: 
;	Status:	Value to write into the sensor's status register
; RETURN VALUE
;	none
; SIDE EFFECTS:
;	Registers are volatile: the A and X registers may be modified
;-----------------------------------------------------------------------------
_SHT1x_WriteStatus:
 SHT1x_WriteStatus:
 	push	A
 
	; send command "Write Status"
 	call	SHT1x_TxStart
 	mov		A, cmdSHT1x_WriteStatus
 	call	SHT1x_TxByte
 	call	SHT1x_WaitForAck
 	
 	; send new value
 	pop		A
  	call	SHT1x_TxByte
 	call	SHT1x_WaitForAck
 
 	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
; char SHT1x_ReadStatus (void)
;
;	Reads the status register of a Sensirion SHT1x sensor
;	The checksum is ignored!  (Sensirion confirmed there is an issue with
;	                           calculating the checksum)
;-----------------------------------------------------------------------------
; ARGUMENTS: 
;	none
; RETURN VALUE
;	value of the status register
; SIDE EFFECTS:
;	Registers are volatile: the X register may be modified
;-----------------------------------------------------------------------------
_SHT1x_ReadStatus:
 SHT1x_ReadStatus:
	; send command "Read Status"
 	call	SHT1x_TxStart
 	mov		A, cmdSHT1x_ReadStatus
 	call	SHT1x_TxByte
 	call	SHT1x_WaitForAck
 	
	;read status register into A
	call	SHT1x_RxByte

 	;skip acknowledge to finish transmission
 	mov reg[SHT1x_DR], D_SCK_HH		;DATA=H, SCK=1
	mov reg[SHT1x_DR], D_SCK_H0   	;DATA=H, SCK=0
	
  	ret
.ENDSECTION


.SECTION
;-----------------------------------------------------------------------------
; void SHT1x_nbMeasure_Start (char Command)
;
;	Trigger the sensor to start a single measurement of either the 
;   relative humidity or temperature.
;	call SHT1x_nbMeasureDone and SHT1x_nbMeasure_Result to finish measurement
;-----------------------------------------------------------------------------
; ARGUMENTS: 
;	Command:	Address and command byte to send. Valid values are:
;					%00000011 (measure temperature) and
;					%00000101 (measure humidity)			
; RETURN VALUE
;	none
; SIDE EFFECTS:
;	Registers are volatile: the A and X registers may be modified
;	CRC8 variable will be modified
;-----------------------------------------------------------------------------
SHT1x_nbMeasure_Start:
_SHT1x_nbMeasure_Start:
	; send "Transmit Start", preserving the command
	push	A
	call	SHT1x_TxStart
	pop		A
	; send command byte	
 	call	SHT1x_TxByte_withCRC
 	call	SHT1x_WaitForAck

	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
; BOOL SHT1x_nbMeasure_Done (void)
;
;	returns true, if the sensor has finished the measurement that was 
;   previously triggered by SHT1x_nbMeasure_Start
; 	call SHT1x_nbMeasure_Result to read the result of the measurement
;-----------------------------------------------------------------------------
; ARGUMENTS: 
;	none
; RETURN VALUE
;	TRUE, if measurement has finished
; SIDE EFFECTS:
;	Registers are volatile: the A and X registers may be modified
;-----------------------------------------------------------------------------
SHT1x_nbMeasure_Done:
_SHT1x_nbMeasure_Done:
 	; test if the sensor has finish the measurement
 	tst		reg[SHT1x_DR], SHT1x_DATA_Mask
	jz		.done
.busy:
	mov		A, 0	; return false
	ret
.done:
	mov		A, 1	; return true
	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
; unsigned int SHT1x_nbMeasure_Result (void)
;
;	returns the result of a measurement, after
;   ...the measurement was triggered by SHT1x_nbMeasure_Start
;   ...the measurement has finished (test with SHT1x_nbMeasure_Done)
;-----------------------------------------------------------------------------
; ARGUMENTS: 
;	none
; RETURN VALUE
;	measurement result (depending on the command sent at SHT1x_nbMeasure_Start)
; SIDE EFFECTS:
;   Global variable CRC8 will be modified (CRC8 will be '0' if checksum is correct)
;-----------------------------------------------------------------------------
SHT1x_nbMeasure_Result:
_SHT1x_nbMeasure_Result:
	; read measurement value MSB
	call	SHT1x_RxByte_withCRC
	call	SHT1x_TxAck
	push	A
	
	;read measurement value LSB
	call	SHT1x_RxByte_withCRC
	call	SHT1x_TxAck
	push	A

	;read CRC -> should turn the CRC8 variable back to %00000000
	call	Mirror_CRC8
	call	SHT1x_RxByte_withCRC
 	;skip acknowledge
 	mov reg[SHT1x_DR], D_SCK_HH		;DATA=H, SCK=1
	mov reg[SHT1x_DR], D_SCK_H0		;DATA=H, SCK=0

	; reload LSB into A register , MSB into X (return value) 
	pop		A
	pop		X

	ret
.ENDSECTION
